function [] = betterFollowRightWall(port)
%betterFollowRightWall This is the main program sent to the robot
%   The algorithm is as explained on the wiki and the code below

tstart = tic; %recording the time, to set a rough stopping point for testing
ElapsedTime=0;
currentCheckpoint = 3; %should start from 1, so it can reach the checkpoints in order
SetFwdVelRadiusRoomba(port, .2,inf);
disp('program started');
%main loop to control the robot
while(ElapsedTime<30)
    %updating time
    ElapsedTime=ElapsedTime+toc(tstart);
    disp(ElapsedTime);
    tstart = tic;
    
    %reading bump, wall and checkpoint sensors
    %the checkpoint sensor is the camera
    %findTape uses the camera to identify a piece of tape on the wall
    [bumpr, bumpl,wdr, wdl, wdc, bumpf] = BumpsWheelDropsSensorsRoomba(port);
    bumped= bumpl||bumpf||bumpr;
    wall = WallSensorRoomba(port);
    checkpoint= findTape();
     
    if(bumped==1) %hit a wall, so turn in place till not bumping
        disp('bumped');
        while(bumped==1) %read sensors and line up on the wall
        [bumpr, bumpl,wdr, wdl, wdc, bumpf] = BumpsWheelDropsSensorsRoomba(port);
        bumped= bumpl||bumpf||bumpr;
        wall = WallSensorRoomba(port);
        disp('correcting for bump');
        SetDriveWheelsCreate(port, .035, -.035); %turning in place
        if(wall==1)
            disp('aligned to wall');
            bumped=0;
            SetFwdVelRadiusRoomba(port, .2,inf); %go straight when aligned
        end
        
        end
    end
    
    if(checkpoint==1) % we detected a checkpoint(i.e. tape on the wall)
        %now depending on what the currentCheckpoint is, perform a hard
        %coded task, such as entering a room or skipping a hallway
        SetFwdVelRadiusRoomba(port, 0,0); %stop
        tempstring = sprintf('Found checkpoint number %d', currentCheckpoint);
        disp(tempstring);
        
        %Each case in this switch refers to a specific hard-coded task that
        %the robot will perform based on its location (which checkpoint it
        %is at)
        switch(currentCheckpoint)
            case 1
                disp('Entering Room 2209');
                enterRoom2209(port);
                currentCheckpoint= currentCheckpoint+1;
                tempstring = sprintf('Looking for Checkpoint number %d', currentCheckpoint);
                disp(tempstring);
            case 2
                disp('Entering Room 2187');
                %enterRoom2187(port);
                currentCheckpoint= currentCheckpoint+1;
                tempstring = sprintf('Looking for Checkpoint number %d', currentCheckpoint);
                disp(tempstring);
            case 3    
                disp('Hallway checkpoint 1, passing gaps...');
                %skipCheckpoint3(port);
                currentCheckpoint= currentCheckpoint+1;
                tempstring = sprintf('Looking for Checkpoint number %d', currentCheckpoint);
                disp(tempstring);
        end
        
    end
    
    if(wall==0) %lost wall, arc towards wall
        disp('lost wall');
        while(bumped==0)
            [bumpr, bumpl,wdr, wdl, wdc, bumpf] = BumpsWheelDropsSensorsRoomba(port);
            bumped= bumpl||bumpf||bumpr;
            disp('arcing till bump');
            SetFwdVelRadiusRoomba(port, .2, -1); %arcing towards wall
        end
    end
    
end

        
%ending the program, making sure robot is stopped, displaying how much time
%has elapsed
disp('program ended');
disp(ElapsedTime);
           SetFwdVelRadiusRoomba(port, 0,0); %stop



end

