%This function process an image taken from the webcam of a classroom. It
%first detects where the windows are based on value (lowest value, since
%we're looking at windows at night) in the hsv color space. Then it
%determines the connected components of window-looking pixels, and labels
%the connected components over the size of the window threshold. Now we
%analyze each window--looking for the 'colorToLookFor' within the frame.
%(colorToLookFor is an h value in hsv. the current is .95, real close to
%pure red. we're using hot pink tape in this instance). We then measure the
%distance between the pixel closest in color to colorToLookFor within the
%current window region, and if the distance is greater than the open window
%threshold, we consider the window open. We return an array of
%booleans--one for each window object in the input image--where 0 is closed
%and 1 is an open window.

function isOpen = findOpenWindowNight()
%First we define the color we're looking for and then we grab the image to
%analyze.
done = 0;
colorToLookFor=.95;
while(done ==0)
    try
    imageIn = imread('/Users/cylala/Desktop/r2d2.jpg');
    image(imageIn);
    done=1;
    catch
    disp('Could not read image file, trying again');    
    end
end

%change to hsv color space
% tic;
imageOfWindow = rgb2hsv(imageIn);
imageOfWindow(:,:,3) = ones(size(imageOfWindow(:,:,3))) -...
    imageOfWindow(:,:,3);

%Round off the value--the window should be dark enough that it is
%distinguishable. If there are problems finding window regions, try playing
%with the +/- at the end of the statement.
imageOfWindow(:,:,3) = round(imageOfWindow(:,:,3)-.4);

%Create a mask for the image. That is, consider all pixels that are either
%too high or too low in the frame to be a window non-window pixels.--might
%need to adjust height once it's mounted. We also cut off a few rows on the
%sides, strictly for implementation reasons (makes our connected component
%algorithm easier to execute).
imageOfWindow(1:200,:,:) = zeros(size(imageOfWindow(1:200,:,:)));
imageOfWindow(408:end,:,:) = zeros(size(imageOfWindow(408:end,:,:)));
imageOfWindow(:,1:3,:) = zeros(size(imageOfWindow(:,1:5,:)));
imageOfWindow(:,end-2:end,:) = zeros(size(imageOfWindow(:,end-4:end,:)));
%convert to bw, essentially
imageOfWindow = imageOfWindow(:,:,3);
% windowKernel = ones(10, 10);

%These while loops will go through all the pixels and find every connected
%component. Right now, a component is considered connected to its neighbor
%if it's within a 5 square pixel radius. the connected components are
%destinguished by an integer value.
iWindowComp = 1;
x = 1;
y = 1;
compWindow = zeros(size(imageOfWindow));
while x < size(imageIn,2)
    while y < size(imageIn,1)
        %if the pixel at y,x was classified as a window pixel, then we look
        %through the pixels that have already been analyzed within a 5
        %pixel radius. if any of them are already classified as a window
        %connected component, then we make the current pixel and any other
        %foreground pixels in the 5 pixel radius members of the connected
        %component with the lowest number.
        if imageOfWindow(y, x) == 1 
            connectedKernel = [compWindow(y-5:y, x-5:x); ...
                compWindow(y+1, x-5:x-1) 0;...
                compWindow(y+2, x-5:x-1) 0;...
                compWindow(y+3, x-5:x-1) 0;...
                compWindow(y+4, x-5:x-1) 0;...
                compWindow(y+5, x-5:x-1) 0;];
            if max(max(connectedKernel)) == 0;
                compWindow(y, x) = iWindowComp;
                iWindowComp = iWindowComp+1;%if the pixel begins new concom
            elseif min(min(connectedKernel(connectedKernel>0))) == max(max(connectedKernel));
                compWindow(y, x) =...
                    min(min(connectedKernel(connectedKernel>0)));
            else
                compWindow(y, x) =...
                    min(min(connectedKernel(connectedKernel>0)));
                for i = min(min(connectedKernel(connectedKernel>0)))...
                        +1:max(max(connectedKernel))
                    compWindow(compWindow == i) =...
                        min(min(connectedKernel(connectedKernel>0)));
                end
            end
        end
        y = y+1;
    end
    y = 1;
    x = x+1;
end

%Now we get rid of all the tiny connected components, so we're only looking
%at the connected components within the frame that are actually windows.
%The connected componenet must have at least 200 pixels to be considered
%window sized. Most windows are much bigger than this, but close blinds can
%make the connected region much smaller. We renumber the window regions
%starting from 1 and incrementing by 1. if everything's working thus far,
%we should have 4 window regions.
iRegions = 1;
for i = min(min(compWindow(compWindow > 0))):max(max(compWindow))
    if numel(compWindow(compWindow == i)) > 200
        compWindow(compWindow == i) = iRegions;
        iRegions = iRegions +1;
    else
        compWindow(compWindow == i) = 0;
    end 
end


%And finally we see where the tape is in the photo and measure its distance
%from the bottom of the window region. The tape is found by comparing the
%hue of every pixel within the window region (note:this means within the
%bounds of the connected component, not necessarily part of the connected
%component itself) to the colorToLookFor hue value. Once we have found the
%location of this pixel, we simply see how far it is from the pixel in the
%window region with the greatest y-value (the farthest down on the frame),
%and if it's farther than the threshold, the window is open.
% closestColor = uint8(zeros(1, 1, 3));
isOpen = zeros(1, iRegions-1);
for i = 1:iRegions-1
    [rows cols] = find(compWindow == i);
    minCol = min(cols);
    maxCol = max(cols);
    minRow = min(rows);
    maxRow = max(rows);
    rows = zeros(1,(maxRow-minRow)*(maxCol-minCol));
    cols = zeros(size(rows)); %rows and cols form the window region
    for iCol = 1:(maxCol-minCol)
        rows((iCol-1)*(maxRow-minRow)+1:iCol*(maxRow-minRow)+1) = minRow:maxRow;
    end
    for iRow = 1:(maxRow-minRow)
        cols((iRow-1)*(maxCol-minCol)+1:iRow*(maxCol-minCol)+1) = minCol:maxCol;
    end
    bottomBox = max(rows);%the bottom of the window region

    closestColor = rgb2hsv(imageIn(rows(1), cols(1), :));
    closestColor = closestColor(1);
    tapeIndex = [rows(1), cols(1)];%start with position 1
    for j = 1:numel(rows)
        testerColor = rgb2hsv(imageIn(rows(j), cols(j), :));
        testerColor = testerColor(1);
        if abs(closestColor - colorToLookFor) > ...
                abs(testerColor - colorToLookFor)
            closestColor = testerColor;
            tapeIndex = [rows(j), cols(j)];
        end
    end%iterate through all pixels, finding closest hue
    if bottomBox - tapeIndex(1) > 40%current threshold
%         fprintf('window %1.0f is open!\n', i);
        isOpen(i) = 1;
    end
end


%Some simple ways to display the photos for debugging
% conver back so we can look at it.
% uhImage = zeros(size(imageIn));
% for i = 1:3
%     uhImage(:,:,i) = imageOfWindow;
% end
% image(uhImage);
% openness = 0;
% image(compWindow);
% toc;
end