%findTape determines when the robot is at a checkpoint, which corresponds
%to either a doorway or some kind of obstacle we have to navigate around.
%It will find a checkpoint by determining the existence of hot pink tape on
%the wall directly to the side of the robot. If the tape is there, we're at
%a checkpoint.

function [dursTape] = findTape()
% tic;

%Define the tape range--this is hot pink in hallway light.
tapeRange = [150 180; 2 12; 30 50];

%Grab the image from where the webcam saves it.
try
    imageOfWindow = imread('/Users/cylala/Desktop/r2d2.jpg');
catch
    disp('Could not read image file');
    dursTape=-1;
    return;
end

dursTape = 0;
classifiedImage = uint8(zeros(size(imageOfWindow(:,:,1))));

%Working through all the pixels in the image, determine whether or not the
%pixel's rgb is in the hot pink tape range. classifiedImage is a big
%frame-sized matrix of 0s and 1s, 1s being tape pixels and 0s everything
%else.
for i = 1:numel(imageOfWindow(:,1,1))
    for j = 1:numel(imageOfWindow(1,:,1))
        if imageOfWindow(i,j,1)>=tapeRange(1,1)-5&&imageOfWindow(i,j,1)<=tapeRange(1,2)+5
            if imageOfWindow(i,j,2)>=tapeRange(2,1)-5&&imageOfWindow(i,j,2)<=tapeRange(2,2)+5
                if imageOfWindow(i,j,3)>=tapeRange(3,1)-5&&imageOfWindow(i,j,3)<=tapeRange(3,2)+5
                    classifiedImage(i,j) = 1;
                end
            end
        end
    end
    
end

%Threshold for how much tape we have to see. If there are at least 500 tape
%pixels, well then we lookin at some tape.
if sum(sum(classifiedImage))>500
    dursTape = 1;
end
% toc;
end